#!/usr/bin/env python

import pygtk,sys
pygtk.require('2.0')
import gtk, gobject, pango

from xml.dom.ext.reader.Sax import FromXmlFile, FromXmlStream
from xml.dom.minidom import parseString
from xml.dom import minidom, Node

class XMLTreeView(gtk.TreeView):
    def __init__(self):
        gtk.TreeView.__init__(self)
        self.__init_model()
        self.__init_view_columns()
        self.connect('button_press_event',self.button_press_callback)
        self.popupMenu = None

    def __init_model(self):
        store = gtk.TreeStore(gobject.TYPE_PYOBJECT,str)
        self.set_model(store)
        self.set_headers_visible(False)

    def __init_view_columns(self):
        col = gtk.TreeViewColumn()
        col.set_title('Tree')
        render_text = gtk.CellRendererText()
        render_text.connect('edited', self.cell_edited)
        render_text.set_property('editable', True)
        col.pack_start(render_text, expand=True)
        col.add_attribute(render_text, 'text', 1)
        self.append_column(col)

    def button_press_callback(self, view, event):
      if event.button == 3:
        x = int(event.x)
        y = int(event.y)
        time = event.time
        path, col, cellx, celly = self.get_path_at_pos(x, y)
        self.grab_focus()
        self.set_cursor( path, col, 0)
        if self.popupMenu is not None:
          self.popupMenu.popup( None, None, None, event.button, time)
        return True
      return False
    
    def append_tag(self, aParent, aTag):
        store = self.get_model()
        stringForTag = "%s" % aTag.__class__
        if aTag.nodeType is Node.DOCUMENT_NODE:
          for childNode in aTag.childNodes:
            self.append_tag(aParent, childNode)
          return
        if aTag.nodeType is Node.DOCUMENT_TYPE_NODE:
          return
        if aTag.nodeType is Node.TEXT_NODE:
          return
        if aTag.nodeType is Node.ELEMENT_NODE:
          stringForTag = aTag.nodeName
        tagNode = store.append(aParent,[aTag,stringForTag])
        for childNode in aTag.childNodes:
          self.append_tag(tagNode, childNode)

    def cell_edited(self, cell, path, newLabel):
      iter = self.get_model().get_iter(path)
      tag = self.get_model().get_value(iter, 0)
      #tag._set_nodeName = newLabel
      print tag
      
    def set_document(self, aXMLDocument):
      store = self.get_model()
      rootNode = store.append(None, [aXMLDocument,"XML"])
      self.append_tag(rootNode,aXMLDocument)
      

class XMLPropertiesView(gtk.TreeView):
    def __init__(self):
      gtk.TreeView.__init__(self)
      self.__init_model()
      self.__init_columns()

    def __init_model(self):
        store = gtk.ListStore(str,str)
        self.set_model(store)
        self.set_headers_visible(True)

    def __init_columns(self):
        colName = gtk.TreeViewColumn()
        colName.set_title('Name')
        render_text_colName = gtk.CellRendererText()
        colName.pack_start(render_text_colName, expand=False)
        colName.add_attribute(render_text_colName, 'text', 0)
        self.append_column(colName)

        colValue = gtk.TreeViewColumn()
        colValue.set_title('Value')
        render_text_colValue = gtk.CellRendererText()
        colValue.pack_start(render_text_colValue, expand=True)
        colValue.add_attribute(render_text_colValue, 'text', 1)
        self.append_column(colValue)

    def show_for(self,aTagNode):
      store = self.get_model()
      store.clear()
      if aTagNode.attributes is None:
        return
      for (key, attribute) in aTagNode.attributes.items():
        store.append( [key, attribute.value])

class XMLTextView(gtk.TextView):
    def __init__(self):
      gtk.TextView.__init__(self)
      self.font_desc = pango.FontDescription('monospace')
      self.modify_font(self.font_desc)
      
    def show_for(self, aTagNode):
      self.get_buffer().set_text("")
      if aTagNode.nodeType is not Node.ELEMENT_NODE:
        return
      if len(aTagNode.childNodes) != 1:
        return
      if aTagNode.childNodes[0].nodeType is not Node.TEXT_NODE:
        return
      self.get_buffer().set_text(aTagNode.childNodes[0].data)


class XMLBorwser:
    def __init__(self,xmlFileName):
      self.__init_view()
      self.read_xml_file(xmlFileName)
    
    def __init_view(self):
      win = gtk.Window()
      win.set_title('XML Browser')
      win.set_size_request(width=400, height=300)
      win.connect('destroy', gtk.mainquit)
      sw = gtk.ScrolledWindow()
      self.tree = XMLTreeView()
      sw.add(self.tree)
      paned = gtk.VPaned()
      paned.pack1(sw)

      self.notebook = gtk.Notebook()
      self.properties = XMLPropertiesView()
      self.notebook.append_page(self.properties,gtk.Label("Attributes"))

      self.fullText = XMLTextView()
      self.notebook.append_page(self.fullText,gtk.Label("Text"))
      paned.pack2(self.notebook)
      win.add(paned)
      
      win.show_all()
      
      self.tree.connect("row-activated", self.show_properties_for_path)
      self.tree.get_selection().connect("changed", self.show_properties_for_selection)

      self.tree.popupMenu = self.build_popup_menu()
      #self.tree.connect("",self.popupMenu)

    def build_popup_menu(self):
      submenu = gtk.Menu()
      
      quitItem = gtk.MenuItem("quit")
      quitItem.connect("activate", gtk.mainquit, "quit")
      
      #swapItem = gtk.MenuItem("swap")
      #swapItem.connect("activate", self.action_pressOnButton_1, "swap")
      
      #updateItem = gtk.MenuItem("update")
      #updateItem.connect("activate", self.action_pressOnButton_2, "update")
      
      #exploreItem = gtk.MenuItem("explore")
      #exploreItem.connect("activate", explore)
      
      #submenu.add(swapItem)
      #submenu.add(updateItem)
      #submenu.add(exploreItem)
      #submenu.add(gtk.SeparatorMenuItem())
      submenu.add(quitItem)
      submenu.show_all()
      return submenu

    def show_properties_for_selection(self, selection):
      model, iter = selection.get_selected()
      tag = model.get_value(iter, 0)
      self.show_properties_for_tag(tag)
      
    def show_properties_for_path(self, treeview, path, column, user_data=None):
      iter = self.tree.get_model().get_iter(path)
      tag = self.tree.get_model().get_value(iter, 0)
      self.show_properties_for_tag(tag)
      
    def show_properties_for_tag(self, aTag):
      self.properties.show_for(aTag)
      self.fullText.show_for(aTag)

    def read_xml_file(self,aFilename):
        doc = FromXmlFile(aFilename)
        self.tree.set_document(doc)
        
        
      
if __name__ == '__main__':
    browser = XMLBorwser("input.xml")
    gtk.mainloop()
